﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PruferCode
{
    public class Prufer
    {
        public static string TreeRecovering(int[] prufersCode, int[] treeVertices)
        {
            string tree = "";
            int pruferLenght = prufersCode.Length;
            int n = pruferLenght + 2;

            int[] edges = new int[n + 1];

            foreach (int vertic in treeVertices) 
            {
                edges[vertic] = 1;
            }

            foreach (int code in prufersCode) 
            {
                edges[code]++;
            }

            for (int i = 0; i < pruferLenght; i++)
            {
                int u = prufersCode[i];
                int v = treeVertices.FirstOrDefault(x => edges[x] == 1);

                edges[u] = v;
                tree += $"{u} {v}, ";

                edges[u]--;
                edges[v]--;
            }

            int[] remaininEdges = treeVertices.Where(x => edges[x] == 1).ToArray();
           
            if (remaininEdges.Length == 1)
            {
                tree += $"{remaininEdges[0]}";
            }
            else if (remaininEdges.Length == 2)
            {
                tree += $"{remaininEdges[0]} {remaininEdges[1]}";
            }
            
            return tree;
        }

        public static int[] TreeToPrufer(int[,] edges, int n)
        {
            int[] degree = new int[n];
            for (int i = 0; i < edges.GetLength(0); i++)
            {
                degree[edges[i, 0]]++;
                degree[edges[i, 1]]++;
            }

            int[] prufersCode = new int[n - 2];
            int index = 0;

            while (index < n - 2) 
            {
                int leaf = FindLeaf(degree);
                int neighbor = FindNeighbor(leaf, edges);
                prufersCode[index++] = neighbor;

                degree[leaf]--;
                degree[neighbor]--;

                RemoveEdge(ref edges, leaf);
            }

            return prufersCode;
        }

        private static int FindLeaf(int[] degree)
        {
            for (int i = 0; i < degree.Length; i++) 
            {
                if (degree[i] == 1)
                {
                    return i;
                }
            }
            return -1;
        }

        private static int FindNeighbor(int leaf, int[,] edges)
        {
            for (int i = 0; i < edges.GetLength (0); i++)
            {
                if (edges[i, 0] == leaf)
                {
                    return edges[i, 1];
                }
                if (edges[i, 1] == leaf)
                {
                    return edges[i, 0];
                }
            }
            return -1;
        }

        private static void RemoveEdge(ref int[,] edges, int leaf)
        {
            for (int i = 0; i < edges.GetLength(0); i++) 
            {
                if (edges[i, 0] == leaf || edges[i, 1] == leaf)
                {
                    edges[i, 0] = -1;
                    edges[i, 1] = -1;
                    break;
                }
            }
        }
    }
}
